C**********************************************************************
C
C ===> onetv.f
C 
C Program to calculate value and policy functions for the 
C paper "Learning and Control in a Changing Economic
C Environment" (with univariate normal beliefs about the unknown slope
C of the Phillips curve and time-varying parameter)
C
C A description of the structure of the algorithm and
C the used parameters and variables is given in the readme.txt
C file available under "http://www.volkerwieland.com/publications.htm".
C
C
C Programs and routines used are:
C ===============================
C
C Name    | Purpose                                      
C --------------------------------------------------------------
C onetv     main program, calls the subroutines
C files     subroutine, open files
C input     subroutine, reads in parameters from 
C           onetv.in
C init      subroutine, initializes grids for state 
C           variables
C value     subroutine, performs values iterations
C output    subroutine, writes approximated value and 
C           policy functions to files onetv.val and 
C           onetv.pol
C f(.)      function, right-hand side of the functio-
C           nal equation
C eval      subroutine, evaluates right-hande side of 
C           the functional equation
C locate    subroutine, implements a search on a grid 
C           for the two gridpoints closest to a given 
C           value
C golden    function, golden section search for mini-
C           mum
C
C Program written by Volker Wieland
C
C Last Change:  2/3/03
C      
C**********************************************************************
C 
       PROGRAM  onetv
C
       IMPLICIT DOUBLE PRECISION (A-H,O-Z)
       PARAMETER(IPG1=26,IPG2=21,IPG3=26)
       DOUBLE PRECISION DELTA,OMEGA,SIGMA,SIGMAETA,RHO
       DOUBLE PRECISION TARGET,ADJUST
       DOUBLE PRECISION V(IPG1,IPG2,IPG3,2)
       DOUBLE PRECISION H(IPG1,IPG2,IPG3,2)
       DOUBLE PRECISION PG1(IPG1),PG2(IPG2),PG3(IPG3)
       DOUBLE PRECISION RNO(100),WEI(100)
       INTEGER ITVAL,ITPOL,ITPOLY,IADJUST,ITER,INTER
C        
       COMMON /COEF/ DELTA,XUPP,XLOW,OMEGA,SIGMA,SIGMAETA
     X               ,RHO,TARGET,ADJUST
       COMMON /TV/ V,H
       COMMON /GRD/ PG1,PG2,PG3
       COMMON /INT/ RNO,WEI
       COMMON /NIT/ ITVAL,ITPOL,ITPOLY,IADJUST,ITER,INTER
C
C
C
       WRITE(*,*)
       WRITE(*,*)
       WRITE(*,*)
       WRITE(*,*) '  CALCULATION OF THE VALUE FUNCTION FOR LEARNING AND' 
       WRITE(*,*) '  =================================================='
       WRITE(*,*)
       WRITE(*,*) '      CONTROL IN A CHANGING ECONOMIC ENVIRONMENT'
       WRITE(*,*) '      =========================================='
       WRITE(*,*)
C
       CALL FILES
C
       CALL INPUT
C      
       CALL INIT
C    
       CALL VALUE
C       
       CALL OUTPUT
C
       WRITE(*,*)
       WRITE(*,*) 'END OF PROGRAM' 
C
       END
C
C       
C***********************************************************************
C       
       SUBROUTINE FILES
C
C      WRITE(*,*) 'SUBROUTINE TO OPEN FILES'
C      WRITE(*,*)
C
C onetv.in contains all input parameters to be chosen by the user
C to define the problem. 
C int100.dat contains nodes and weights for 100-point Gaussian quadrature. 
C onetv.pol contains optimal policy evaluated at the grid points
C onetv.val contains results for value functions evaluated at grid points
C
C
       OPEN(UNIT=8, FILE='onetv.in') 
       OPEN(UNIT=9, FILE='int100.dat')
       OPEN(UNIT=10, FILE='onetv.pol') 
       OPEN(UNIT=11, FILE='onetv.val')
C
       RETURN
C       
       END
C       
C***********************************************************************
C
       SUBROUTINE INPUT
C
       IMPLICIT DOUBLE PRECISION (A-H,O-Z)
       PARAMETER(IPG1=26,IPG2=21,IPG3=26)
       DOUBLE PRECISION DELTA,OMEGA,SIGMA,SIGMAETA,RHO
       DOUBLE PRECISION TARGET,ADJUST
       DOUBLE PRECISION RNO(100),WEI(100)
       INTEGER ITVAL,ITPOL,ITPOLY,IADJUST,ITER,INTER
C       
       COMMON /COEF/ DELTA,XUPP,XLOW,OMEGA,SIGMA,SIGMAETA
     X               ,RHO,TARGET,ADJUST
       COMMON /INT/ RNO,WEI
       COMMON /NIT/ ITVAL,ITPOL,ITPOLY,IADJUST,ITER,INTER
C
C
C       WRITE(*,*) 'SUBROUTINE FOR PARAMETER INPUT'
C       WRITE(*,*)
C
C Routine input reads parameters chosen by the user from onetv.in. 
C These consist of:
C i)   the discount factor (delta)
C ii)  the relative weight of the deviation of the control variable u from
C       its target (omega)
C iii) the variance of shocks to the state variable (sigmaeps)
C iv)  the target for the state variable (target)
C v)   the weight of the deviation of the state variable x from its target (rho)
C vi)  the variance of the innovation to beta (sigmaeta)
C vii) maximum numbers for value and policy function iterations (itval,itpol)
C viii)additional indicator variables (inter,itpoly,iadjust)
C
        READ(8,*) DELTA
        READ(8,*) XUPP, XLOW
        READ(8,*) OMEGA
        READ(8,*) SIGMA
        READ(8,*) TARGET
        READ(8,*) INTER
        READ(8,*) ADJUST
        READ(8,*) ITVAL
        READ(8,*) ITPOLY
        READ(8,*) ITPOL
        READ(8,*) IADJUST
        READ(8,*) RHO
        READ(8,*) SIGMAETA
C
C read in nodes and weights for Gaussian quadrature from 
C int100.dat.
C 
        DO 10  K=1,100
         READ(9,*) RNO(K), WEI(K) 
 10     CONTINUE
C
       RETURN
C       
       END
C       
C***********************************************************************
C
       SUBROUTINE INIT
C
C Subroutine to initialize the grid of state variables and the value
C function at the grid.
C
       IMPLICIT DOUBLE PRECISION (A-H,O-Z)
       PARAMETER(IPG1=26,IPG2=21,IPG3=26)
       DOUBLE PRECISION DELTA,OMEGA,SIGMA,SIGMAETA,RHO
       DOUBLE PRECISION TARGET,ADJUST
       DOUBLE PRECISION V(IPG1,IPG2,IPG3,2)
       DOUBLE PRECISION H(IPG1,IPG2,IPG3,2)
       DOUBLE PRECISION PG1(IPG1),PG2(IPG2),PG3(IPG3)
       INTEGER ITVAL,ITPOL,ITPOLY,IADJUST,ITER,INTER
       COMMON /NIT/ ITVAL,ITPOL,ITPOLY,IADJUST,ITER,INTER
C       
       COMMON /COEF/ DELTA,XUPP,XLOW,OMEGA,SIGMA,SIGMAETA
     X               ,RHO,TARGET,ADJUST
       COMMON /TV/ V,H
       COMMON /GRD/ PG1,PG2,PG3       
C
C      WRITE(*,*) 'SUBROUTINE FOR VALUE FUNCTION INITIALIZATION'
C      WRITE(*,*)
C
C
C define the grids for the state variables pg
C
C
C Define the grid-points for the 3 state variables:
C PG1: mean of the slope
C PG2: variance of the slope 
C PG3: lagged endogenous variable
C
       P2MIN=0.01d0
       P2MAX=1.0d0
       P3MIN=-2.0D0
       P3MAX=2.0D0
       P1MIN=-2.0d0
       P1MAX=2.0d0
C
C
C Grid for mean of beta
C
       P1INIT=P1MIN
       IPG1MIDL=IPG1/2
       IPG1MIDL1=IPG1MIDL-1
       IPG1MIDU=IPG1MIDL+1
       IPG1MIDU1=IPG1MIDU+1
       P1INCR=(P1MAX)/(REAL(IPG1MIDL)-3.0d0)
C
C
       PG1(1)=-5.0D0
       PG1(2)=-3.0D0
       DO 20 J=3,IPG1MIDL1
          PG1(J)=P1INIT
          P1INIT=P1INIT+P1INCR
 20    CONTINUE
       PG1(IPG1MIDL)=-0.05D0
       PG1(IPG1MIDU)=+0.05D0
       P1INIT=0.0D0
       IPG11=IPG1-1
       IPG12=IPG1-2
       DO 21 J=IPG1MIDU1,IPG12
          P1INIT=P1INIT+P1INCR
          PG1(J)=P1INIT
 21    CONTINUE
       PG1(IPG11)=3.0D0
       PG1(IPG1)=5.0D0
C
C
C Grid for variance of beta
C
       P2INIT=0.0d0
       IPG21=IPG2-1
       P2INCR=(P2MAX)/(REAL(IPG2)-1.0d0)
C
       PG2(1)=P2MIN**2
       DO 22 J=2,IPG21
         P2INIT=P2INIT+P2INCR
         PG2(J)=P2INIT**2
 22    CONTINUE
       PG2(IPG2)=P2MAX**2
C
C
C Grid for lagged state variable
C
       P3INIT=P3MIN
       IPG32=IPG3-2
       IPG3MIDL=IPG3/2
       IPG3MIDL1=IPG3MIDL-1
       IPG3MIDU=IPG3MIDL+1
       IPG3MIDU1=IPG3MIDU+1
       P3INCR=(P3MAX)/(REAL(IPG3MIDL)-3.0d0)
c
        PG3(1)=-5.0D0
        PG3(2)=-3.0D0
       DO 30 J=3,IPG3MIDL1
          PG3(J)=P3INIT
          P3INIT=P3INIT+P3INCR
 30    CONTINUE
C
C
      PG3(IPG3MIDL)=-0.0001D0
      PG3(IPG3MIDU)=+0.0001D0
       P3INIT=0.0D0
        DO 31 J=IPG3MIDU1,IPG32
          P3INIT=P3INIT+P3INCR
          PG3(J)=P3INIT
 31    CONTINUE
        IPG31=IPG3 - 1
C
        PG3(IPG31)=3.0D0
        PG3(IPG3)=5.0D0
C
C
C
C checking whether the grids chosen for the state variables are ok.
C
C        DO 50 I=1,IPG1
C           WRITE(*,*) 'PG1R ',I, PG1(I)
C 50    CONTINUE
C
C        DO 51 I=1,IPG2
C           WRITE(*,*) 'PG2R ',I, PG2(I)
C 51    CONTINUE
C
C        DO 52 I=1,IPG3
C           WRITE(*,*) 'PG3R ',I, PG3(I)
C 52     CONTINUE
C
C Initialize the value function: As the first guess needed for the
C iterative method, we take the value obtained by choosing a myopic
C policy which minimizes the expected single-period loss.
C       
C Functional Form:  fill in
C
        DO 60 I1=1,IPG1
         DO 60 I2=1,IPG2
          DO 60 I3=1,IPG3
C
             H0=-((RHO*PG3(I3)-TARGET)*PG1(I1))
     X           /(PG1(I1)**2 + PG2(I2)+SIGMAETA) 
C
             V0= SIGMA + (RHO*PG3(I3)-TARGET)**2 
     X        * (PG2(I2)+ SIGMAETA)
     X          /(PG2(I2)+SIGMAETA+PG1(I1)**2)

C
             HCERT=0.0D0
             VCERT=0.0D0
C
            H(I1,I2,I3,1)=H0
C           IF(IADJUST.EQ.1) THEN
            V(I1,I2,I3,1)= ADJUST + V0/(1-DELTA)
C           ELSE
C            V(I1,I2,I3,1)= V0
C           ENDIF
C
            V(I1,I2,I3,2)=V(I1,I2,I3,1)
            H(I1,I2,I3,2)=H(I1,I2,I3,1) 
C
 60    CONTINUE
C
 62    FORMAT (E16.8)
C
       RETURN
C 
       END
C        
C***********************************************************************
C       
       SUBROUTINE OUTPUT
C
C Subroutine to prepare the output for onetv.pol which will contain the
C approximation to the optimal policy and onetv.val, which will contain
C the approximation to the value function.
C
       IMPLICIT DOUBLE PRECISION (A-H,O-Z)
       PARAMETER(IPG1=26,IPG2=21,IPG3=26)
       DOUBLE PRECISION DELTA,OMEGA,SIGMA,SIGMAETA,RHO
       DOUBLE PRECISION TARGET,ADJUST
       DOUBLE PRECISION V(IPG1,IPG2,IPG3,2)
       DOUBLE PRECISION H(IPG1,IPG2,IPG3,2)
       DOUBLE PRECISION PG1(IPG1),PG2(IPG2),PG3(IPG3)
       DOUBLE PRECISION RNO(100),WEI(100)
       INTEGER ITVAL,ITPOL,ITPOLY,IADJUST,ITER,INTER
C       
       COMMON /COEF/ DELTA,XUPP,XLOW,OMEGA,SIGMA,SIGMAETA
     X               ,RHO,TARGET,ADJUST
       COMMON /TV/ V,H
       COMMON /GRD/ PG1,PG2,PG3
       COMMON /INT/ RNO,WEI
       COMMON /NIT/ ITVAL,ITPOL,ITPOLY,IADJUST,ITER,INTER
C
       WRITE(*,*) 'SUBROUTINE FOR OUTPUT PREPARATION'
       WRITE(*,*)
C
C writing the parameters chosen by the user in onetv.in to the
C output files.
C
C       WRITE(10,*) DELTA, ' DELTA'
C       WRITE(10,*) XUPP, XLOW, ' XUPP, XLOW'
C       WRITE(10,*) OMEGA,SIGMA,SIGMAETA, ' OMEGA,SIGMA,SIGMAETA'
C       WRITE(10,*) IPG1,IPG2,IPG3, ' IPG1-5'
C       WRITE(10,*) ITVAL , ' ITVAL'
C       WRITE(10,*) INTER , ' INTER'
C       WRITE(10,*) ITPOLY , ' ITPOLY'
C       WRITE(10,*) ITPOL , ' ITPOL'
C       WRITE(10,*) IADJUST , ' IADJUST'
C       WRITE(10,*) ADJUST , ' ADJUST'

C
C       WRITE(11,*) DELTA, ' DELTA'
C       WRITE(11,*) XUPP, XLOW, ' XUPP, XLOW'
C       WRITE(11,*) OMEGA,SIGMA,SIGMAETA, ' OMEGA,SIGMA,SIGMAETA'
C       WRITE(11,*) IPG1,IPG2,IPG3, ' IPG1-5'
C       WRITE(11,*) ITVAL , ' ITVAL'
C       WRITE(11,*) INTER , ' INTER'
C       WRITE(11,*) ITPOLY , ' ITPOLY'
C       WRITE(11,*) ITPOL , ' ITPOL'
C       WRITE(11,*) IADJUST , ' IADJUST'
C       WRITE(11,*) ADJUST , ' ADJUST'
C
C calculate H0 and V0 which are the optimal single period policy
C (the myopic policy) and the implied single-period loss respectively.
C They are written to the output files so that they can be compared to
C the approximations to the optimal policy and value function.
C

        DO 20 I1=1,IPG1
         DO 20 I2=1,IPG2
          DO 20 I3=1,IPG3
C
             H0=-((RHO*PG3(I3)-TARGET)*PG1(I1))
     X           /(PG1(I1)**2 + PG2(I2) + SIGMAETA) 
C
             V0= SIGMA + (RHO*PG3(I3)-TARGET)**2 
     X        * (PG2(I2)+SIGMAETA)
     X         /(PG2(I2)+SIGMAETA+PG1(I1)**2)
C
             HCERT=-((RHO*PG3(I3)-TARGET)/PG1(I1))
             VCERT=0.0D0
C
C change output so that only new functions are saved
C             
            WRITE(10,fmt=23) H(I1,I2,I3,2)
            WRITE(11,fmt=23) V(I1,I2,I3,2)
C
 20    CONTINUE
C
 23    FORMAT (E16.8)
C
C
       RETURN
C
       END
C       
C***********************************************************************
C       
       SUBROUTINE VALUE
C
C value function iteration: ITVAL number of iterations, IPG? - size of 
C the grids of state variables in each dimension
C
       IMPLICIT DOUBLE PRECISION (A-H,O-Z)
       PARAMETER(IPG1=26,IPG2=21,IPG3=26)
       DOUBLE PRECISION DELTA,OMEGA,SIGMA,SIGMAETA,RHO
       DOUBLE PRECISION TARGET,ADJUST
       DOUBLE PRECISION V(IPG1,IPG2,IPG3,2)
       DOUBLE PRECISION H(IPG1,IPG2,IPG3,2)
       DOUBLE PRECISION PG1(IPG1),PG2(IPG2),PG3(IPG3)
       DOUBLE PRECISION RNO(100),WEI(100)
       DOUBLE PRECISION AX,BX,CX,XMIN,TOL,GOLDEN
       DOUBLE PRECISION XPRE(20),FPRE(20)
       INTEGER ITVAL,ITPOL,ITPOLY,IADJUST,ITER,INTER
       INTEGER IP1,IP2,IP3
       INTEGER VPASS,PPASS
       INTEGER LOOK
       EXTERNAL F       
       EXTERNAL GOLDEN
C              
       COMMON /COEF/ DELTA,XUPP,XLOW,OMEGA,SIGMA,SIGMAETA
     X               ,RHO,TARGET,ADJUST  
       COMMON /TV/ V,H
       COMMON /GRD/ PG1,PG2,PG3
       COMMON /INT/ RNO,WEI
       COMMON /NIT/ ITVAL,ITPOL,ITPOLY,IADJUST,ITER,INTER
       COMMON /IP/ IP1,IP2,IP3
       COMMON /LLOOK/ LOOK 
C
       WRITE(*,*)
       WRITE(*,*) 'SUBROUTINE FOR VALUE FUNCTION ITERATION'
       WRITE(*,*)
C
C----------------------------------------------------------  
C
C define integer variable for number of iterations completed 
C
       ITER=0
C
C (1ST LOOP LEVEL)  loop for value function iteration
C
       DO 50 LV=1,ITVAL
C
C----------------------------------------------------------  
C
C define the variable used to check convergence of value 
C function iteration (1 means convergence check is passed).
C
       VPASS=1
C
C----------------------------------------------------------  
C
C remember number of iterations already completed
C
       ITER=ITER+1
C
C----------------------------------------------------------  
C  if look=(1,2,3) then different printout levels
C
       LOOK=0
C
C define various printouts
C 
C       IF((I1.EQ.8).AND.(I2.EQ.11).AND.(I3.EQ.9)) THEN
C         LOOK=2         
C       ELSE
C         LOOK=0
C       ENDIF
C
C ---------------------------------------------------------
C
C (2ND LOOP LEVEL) loop over the grid of the 
C  state variables
C
       WRITE(*,*)
       WRITE(*,*) 'VALUE ITERATION: ',LV
       WRITE(*,*)
       DO 100 I1=1,IPG1
        WRITE(*,*) 'I1= ',I1
        DO 100 I2=1,IPG2
         DO 100 I3=1,IPG3
C
C set states for calling the subroutine EVAL to evaluate the
C functional equation
C  
           IP1=I1
           IP2=I2
           IP3=I3
C
         H0=-((RHO*PG3(I3)-TARGET)*PG1(I1))
     X       /(PG1(I1)**2 + PG2(I2)+SIGMAETA) 
C
C Preliminary rough grid search for minimum, evaluate function 
C values around current H, considering H0, AX, CX
C Compare and take the value of the control x  with the 
C smallest function value F(x) as starting point BX, as the 
C midpoint for the Golden Section Search 
C
          XLO=-100.0D0
          XUP=100.0D0  
C
          XBOUND=-((RHO*PG3(I3)-TARGET)*PG1(I1))/(PG1(I1)**2 + 0.0001) 
c
          IF(XBOUND.LT.0.0D0) THEN
            AX=XBOUND + XLO
           CX=-XBOUND + XUP
          ELSE
           AX=-XBOUND + XLO
           CX=XBOUND + XUP
         ENDIF
C 
         IF((CX.LT.H0).OR.(AX.GT.H0)) THEN
           WRITE(*,*) 
           WRITE(*,*) 'BOUNDS TO0 HIGH OR LOW: H0,AX,CX: ',H0,AX,CX
           WRITE(*,*) 'XBOUND',XBOUND
           WRITE(*,*)
         ENDIF
C
          XPRE(1)=AX
           X=XPRE(1)
           FPRE(1)=F(X)
           XPRE(2)=-2.0D0+H(I1,I2,I3,1)
           X=XPRE(2)
           FPRE(2)=F(X)
           XPRE(3)=-1.5D0+H(I1,I2,I3,1)
           X=XPRE(3)
           FPRE(3)=F(X)
           XPRE(4)=-1.0D0+H(I1,I2,I3,1)
           X=XPRE(4)
           FPRE(4)=F(X)
           XPRE(5)=-0.75D0+H(I1,I2,I3,1)
           X=XPRE(5)
           FPRE(5)=F(X)
           XPRE(6)=-0.5D0+H(I1,I2,I3,1)
           X=XPRE(6)
           FPRE(6)=F(X)
           XPRE(7)=-0.25D0+H(I1,I2,I3,1)
           X=XPRE(7)
           FPRE(7)=F(X)
           XPRE(8)=-0.125D0+H(I1,I2,I3,1)
           X=XPRE(8)
           FPRE(8)=F(X)
           XPRE(9)=H(I1,I2,I3,1)
           X=XPRE(9)
           FPRE(9)=F(X)
           XPRE(10)=0.125D0+H(I1,I2,I3,1)
           X=XPRE(10)
           FPRE(10)=F(X)
           XPRE(11)=0.25d0+H(I1,I2,I3,1)
           X=XPRE(11)
           FPRE(11)=F(X)
           XPRE(12)=0.5d0+H(I1,I2,I3,1)
           X=XPRE(12)
           FPRE(12)=F(X)
           XPRE(13)=0.75d0+H(I1,I2,I3,1)
           X=XPRE(13)
           FPRE(13)=F(X)
           XPRE(14)=1.0d0+H(I1,I2,I3,1)
           X=XPRE(14)
           FPRE(14)=F(X)
           XPRE(15)=1.5d0+H(I1,I2,I3,1)
           X=XPRE(15)
           FPRE(15)=F(X)
           XPRE(16)=2.0d0+H(I1,I2,I3,1)
           X=XPRE(16)
           FPRE(16)=F(X)
           XPRE(17)=0.0d0
           X=XPRE(17)
           FPRE(17)=F(X)
           XPRE(18)=H0
           X=XPRE(18)
           FPRE(18)=F(X)
           XPRE(19)=CX
           X=XPRE(19)
           FPRE(19)=F(X)
      HCERT =-((RHO*PG3(I3)-TARGET)*PG1(I1))/(PG1(I1)**2 + 0.00000001) 
           XPRE(20)=HCERT
           X=XPRE(20)
           FPRE(20)=F(X)

c         write(*,*)

           SMIN=1
           FMIN=FPRE(1)       
           DO 400 SS =1,20
c            WRITE(*,*) 'XPRE= ',XPRE(SS),' FPRE= ',FPRE(SS)
             FDIF=FPRE(SS)-FMIN
             IF(FDIF.LT.0.0D0) THEN
               FMIN=FPRE(SS)
               SMIN=SS
             ENDIF
 400       CONTINUE
C
C----------------------------------------------------------  
C
C set midpoint for golden section search
C
C
           BX=XPRE(SMIN)
C
C set tolerance and then call golden section search, restart
C at upper and lower band for x, but keep the minimum found
C in the preliminary rough grid search
C
C
c         TOL=1.0E-2
c         TOL=1.0E-6
         TOL=1.0E-3
         V(I1,I2,I3,2)=GOLDEN(AX,BX,CX,F,TOL,XMIN)
         H(I1,I2,I3,2)=XMIN
C
C define various printouts
C 
C 
c       IF((I1.EQ.8).AND.(I2.EQ.11).AND.(I3.EQ.60)) THEN
c         LOOK=2         
c       ELSE
c         LOOK=0
c       ENDIF
C
C----------------------------------------------------------  
C
 500     CONTINUE
C
C----------------------------------------------------------  
C various printouts
c
         IF(LOOK.GE.2) THEN
           WRITE(*,*) 'APPROXIMATE MINIMUM'
           WRITE(*,*) ' XMIN =', XMIN
           WRITE(*,*)
         ENDIF
C
C check if value iteration is decreasing
C
        VOK2=V(I1,I2,I3,2)
        VOK1=V(I1,I2,I3,1)

        IF(IADJUST.EQ.1) THEN
         IF(VOK2.GT.VOK1) THEN
C        LOOK=1
         WRITE(*,*) '****************** PROBLEM *****************'
         WRITE(*,*) 'V1,V2,h0,h'
         WRITE(*,*) I1,I2,I3,V(I1,I2,I3,1)
     X             ,V(I1,I2,I3,2)
         WRITE(*,*) I1,I2,I3,H0,H(I1,I2,I3,1)
     X             ,H(I1,I2,I3,2)
         WRITE(*,*) '****************** PROBLEM *****************'
         WRITE(*,*)            
         ENDIF
        ENDIF
C
C various printouts
C
        IF(LOOK.GE.1) THEN
         WRITE(*,*)          
         WRITE(*,*) 'AX,BX,CX',AX,BX,CX
         WRITE(*,*) 'I1-3,V(I1-3,1),V(I1-3,2)'
         WRITE(*,*) I1,I2,I3,V(I1,I2,I3,1)
     X              ,V(I1,I2,I3,2)
         WRITE(*,*) 'I-3,H0,H1,H2'
         WRITE(*,*) I1,I2,I3,H0,H(I1,I2,I3,1)
     X              ,H(I1,I2,I3,2)
         WRITE(*,*) '******************************************'
         WRITE(*,*)
        ENDIF
C----------------------------------------------------------  

C
C reset look variable to avoid printout for all grid points
C
         LOOK=0
C
C----------------------------------------------------------  
C
C check the convergence criterion 
C
        IF(VPASS.EQ.1) THEN
         CHECKV=DELTA*ABS(V(I1,I2,I3,2)-V(I1,I2,I3,1))/
     X          ((1.0D0-DELTA)*V(I1,I2,I3,1))
         IF(CHECKV.GT.0.005) THEN
          VPASS=0     
          NPV1=I1
          NPV2=I2
          NPV3=I3
          CHECKNPV=CHECKV
         ENDIF
        ENDIF
C
 600   CONTINUE
C
C----------------------------------------------------------  
C
C (2ND LOOP LEVEL END) end loop over pgrid
C
 100   CONTINUE
C
C----------------------------------------------------------  
C
C calculate value functions for the gridpoint were convergence
C criterion failed, 
C and print the values before updating the value function
C just to see what's going on 
C
C
C
         H0NPV=-((RHO*PG3(NPV3)-TARGET)*PG1(NPV1))/(PG1(NPV1)**2 
     X              + PG2(NPV2)+SIGMAETA) 
C
             V0NPV= SIGMA + (RHO*PG3(NPV3)-TARGET)**2 
     X        * (PG2(NPV2)+SIGMAETA)
     X          /(PG2(NPV2)+SIGMAETA+PG1(NPV1)**2)
C
         WRITE(*,*) 'NPV1-3 ',NPV1,NPV2,NPV3,' V0NPV ',
     X             V0NPV, ' V1NPV '
     X            ,V(NPV1,NPV2,NPV3,1), 
     X            ' V2NPV',V(NPV1,NPV2,NPV3,2)
C 
C----------------------------------------------------------  
C
C update the value function
C
C
         DO 110 I1=1,IPG1 
          DO 110 I2=1,IPG2
           DO 110 I3=1,IPG3
               V(I1,I2,I3,1)=V(I1,I2,I3,2)
               H(I1,I2,I3,1)=H(I1,I2,I3,2)
 110     CONTINUE
C
C----------------------------------------------------------  
C
C check whether the value function iteration converged. Print out
C the grid point at which convergence failed
C
         WRITE(*,*)
         WRITE(*,*) 'FINISHED VALUE FUNCTION ITERATION',LV
         IF(VPASS.EQ.1) THEN
           WRITE(*,*)
           WRITE(*,*) 'CONVERGENCE OF THE VALUE FUNCTION'
           WRITE(*,*)
           GOTO 51
           ELSE 
           WRITE(*,*) 'NO PASS'
           WRITE(*,*) 'NPV1-3,CHECKNPV'
           WRITE(*,*) NPV1,NPV2,NPV3,CHECKNPV
           WRITE(*,*)
          ENDIF            
C
C here it is possible to skip the policy function iterations
C and move on to the next value function iteration
C
        IF(ITPOLY.EQ.0) THEN
         GOTO 50 
        ENDIF
C
C----------------------------------------------------------  
C  start policy function iteration if desired
C----------------------------------------------------------  
C
C----------------------------------------------------------  
C
C (1ST LOOP LEVEL) loop for policy function iteration
C
       DO 150 LP=1,ITPOL
C
C define the variable used to check convergence of 
C policy function
C iteration (1 means convergence check is passed).
C
       PPASS=1
C
C remember the number of iterations already completed
C
       ITER=ITER+1
C
C----------------------------------------------------------  
C
C (2ND LOOP LEVEL) loop over the grid of the five 
C  state variables
C
       WRITE(*,*) 'POLICY ITERATIONS:'
       WRITE(*,*)
       DO 200 I1=1,IPG1 
        DO 200 I2=1,IPG2
         DO 200 I3=1,IPG3
C
C set states for calling the subroutine EVAL o evaluate the
C functional equation
C  
         IP1=I1
         IP2=I2
         IP3=I3
C 
C iterate over the functional equation for a given policy. The
C policy is the optimal policy obtained from the last value function
C iteration.
C
         XLAST=H(I1,I2,I3,2)
         X=XLAST
         V(I1,I2,I3,2)=F(X)
C
C----------------------------------------------------------  
C
C check the convergence criterion for the policy function iteration. 
C
        IF(PPASS.EQ.1) THEN
         CHECKP=DELTA*ABS(V(I1,I2,I3,2)-V(I1,I2,I3,1))/
     X         ((1.0D0-DELTA)*V(I1,I2,I3,1))
         IF(CHECKP.GT.0.001) THEN
           PPASS=0
           NPP1=I1
           NPP2=I2
           NPP3=I3
           CHECKNPP=CHECKP     
         ENDIF
        ENDIF
C 
C----------------------------------------------------------  
C (2ND LOOP LEVEL END) end of loop over pgrid
C
 200     CONTINUE
C
C----------------------------------------------------------  
C
C write values for comparison at gridpoint which failed 
C the convergence criterion, just to see what's going on
C
C           WRITE(*,*) V(NPP1,NPP2,NPP3,1), 
C     X           V(NPP1,NPP2,NPP3,2)
C
C----------------------------------------------------------  
C
C update the value function
C
C
         DO 210 I1=1,IPG1 
          DO 210 I2=1,IPG2
           DO 210 I3=1,IPG3
              V(I1,I2,I3,1)=V(I1,I2,I3,2)
 210     CONTINUE
C
C----------------------------------------------------------  
C check whether the policy function iteration converged. Print
C out the gridpoint at which convergence failed.
C
         WRITE(*,*) 'POLIT',LP
         IF(PPASS.EQ.1) THEN
           WRITE(*,*)
           WRITE(*,*) 'POLIT CONVERGENCE',LP
           GOTO 151
         ELSE 
           WRITE(*,*) 'NO PASS'
           WRITE(*,*) 'NPP1-3,CHECKNPP'
           WRITE(*,*) NPP1,NPP2,NPP3,CHECKNPP
           WRITE(*,*) 
         ENDIF                       
C----------------------------------------------------------  
C (1ST LOOP LEVEL END) end of policy iterations
C jump to 151 if convergence has been achieved
C
 150      CONTINUE
 151      CONTINUE
C 
C----------------------------------------------------------  
C (1ST LOOP LEVEL END), end of value iteration, jump to 50
C if policy iterations are to be skipped, jump to 51 if
C value iteration convergence has been achieved
C
 50      CONTINUE
 51      CONTINUE
C
C----------------------------------------------------------  
C
        RETURN
C
        END
C       
C       
C***********************************************************************
C
       DOUBLE PRECISION FUNCTION F(X)
C
C the function F is the right-hand side of the functional equation. It
C has to be minimized for every value function iteration and evaluated at
C the last approximation to the optimal policy for every policy function 
C iteration.
C
       IMPLICIT DOUBLE PRECISION (A-H,O-Z)
       DOUBLE PRECISION X
       DOUBLE PRECISION FEVAL
C  
C call the subroutine EVAL which evaluates the right-hand side of the 
C functional equation.
C
C
       CALL  EVAL(X,FEVAL)
C
       F=FEVAL
C
       RETURN
       END
C
C***********************************************************************
C       
       SUBROUTINE EVAL(X,FEVAL)
C 
C This subroutine evaluates the right-hand side of the functional 
C equation. The following operations are necessary:Bayesian updating
C of the beliefs BUP(which are the state variables pg); the approximation
C of the value function at the updated beliefs by linear piecewise
C approximation; and finally the calculation of an integral by means
C of Gaussian Quadrature. 
C
       IMPLICIT DOUBLE PRECISION (A-H,O-Z)
       PARAMETER(IPG1=26,IPG2=21,IPG3=26) 
       DOUBLE PRECISION DELTA,OMEGA,SIGMA,SIGMAETA,RHO
       DOUBLE PRECISION TARGET,ADJUST
       DOUBLE PRECISION V(IPG1,IPG2,IPG3,2)
       DOUBLE PRECISION H(IPG1,IPG2,IPG3,2)
       DOUBLE PRECISION PG1(IPG1),PG2(IPG2),PG3(IPG3)
       DOUBLE PRECISION RNO(100),WEI(100)
       DOUBLE PRECISION BUP1,BUP2,BUP3
       DOUBLE PRECISION X,FEVAL,VBUP
       INTEGER ITVAL,ITPOL,ITPOLY,IADJUST,ITER,INTER,IP1,IP2,IP3
       INTEGER LOOK
C       
C       
       COMMON /COEF/ DELTA,XUPP,XLOW,OMEGA,SIGMA,SIGMAETA
     X               ,RHO,TARGET,ADJUST
       COMMON /TV/ V,H
       COMMON /GRD/ PG1,PG2,PG3
       COMMON /INT/ RNO,WEI
       COMMON /NIT/ ITVAL,ITPOL,ITPOLY,IADJUST,ITER,INTER       
       COMMON /IP/ IP1,IP2,IP3
       COMMON /LLOOK/ LOOK
C
C         WRITE(*,*) 'SUBROUTINE FOR EVALUATING F(X,P)'
C         WRITE(*,*) 
C  
C The values of states for which F is supposed to be evaluated are stored
C in the common area IP and are given by IP1-3.
C
C----------------------------------------------------------  
C       WRITE(*,*) 'SUBROUTINE FOR EVALUATING F(X,P)'
C       WRITE(*,*)        
C       WRITE(*,*)
C
C First we initialize the integral EV at zero.     
C       
         EV=0.0d0
C
C for a given weight and node calculate (BUP) the posterior
C which feeds into the expected value of next period's
C value function
C
C BUP2 is the update of the variance of the slope 
C
C   Kalman filter
C
         BUP2 =  (PG2(IP2)+SIGMAETA)
     x           -((PG2(IP2)+SIGMAETA)**2 * X**2)
     X          /(SIGMA + (PG2(IP2)+SIGMAETA)*X**2)
C
C
         CALL LOCATE(PG2,IPG2,BUP2,LOC)
C   
         IP21=LOC
         IP22=LOC+1
C
         IF(IP21.EQ.0) THEN
           IP21=1
           IP22=1
           W2=0
          GOTO 10
         ENDIF
         IF(IP21.EQ.IPG2) THEN
           IP22=IPG2
           W2=1
          GOTO 10
         ENDIF
         W2 = (BUP2-PG2(IP21))/(PG2(IP22)-PG2(IP21))
 10      CONTINUE
C
C loop over weights and integrands for given p and x
C and initialize ev (the evaluated integral)
C       
         EV=0.0d0
         DO 100 K=1,100
C
C RM is the expectation of next period's 
C      dependent variable 
C
          RM = RNO(K)*SQRT(2.0D0)
     X         *SQRT(SIGMA+(PG2(IP2)+SIGMAETA)*X**2)
     X             + RHO*PG3(IP3) + PG1(IP1)*X
C
         IF(PG2(IP2).EQ.0) THEN
           BUP1 = PG1(IP1)
         ELSE
C
C BUP1 is the update of the mean of the slope 
C
C  Kalman filter version of updating equation
C
          BUP1 = PG1(IP1) + (PG2(IP2)+SIGMAETA)
     X          /(SIGMA + (PG2(IP2)+SIGMAETA)*X**2)
     X           *((RM-RHO*PG3(IP3))*X - X**2 * PG1(IP1))
C
         ENDIF
C
         CALL LOCATE(PG1,IPG1,BUP1,LOC)
C
         IP11=LOC
         IP12=LOC+1
C
         IF(IP11.EQ.0) THEN
            IP11=1
            IP12=1
            W1=0
          GOTO 120
         ENDIF
         IF(IP11.EQ.IPG1) THEN
            IP12=IPG1
            W1=1
          GOTO 120
        ENDIF
        W1 = (BUP1-PG1(IP11))/(PG1(IP12)-PG1(IP11))
 120     CONTINUE
C
C BUP3 is the expectation of next period's 
C      dependent variable 
C
        BUP3=RM
C
         CALL LOCATE(PG3,IPG3,BUP3,LOC)
C
         IP31=LOC
         IP32=LOC+1
C
         IF(IP31.EQ.0) THEN
            IP31=1
            IP32=1
            W3=0
          GOTO 121
         ENDIF
         IF(IP31.EQ.IPG3) THEN
            IP32=IPG3
            W3=1
          GOTO 121
        ENDIF
        W3 = (BUP3-PG3(IP31))/(PG3(IP32)-PG3(IP31))
 121    CONTINUE
C
C during the first value iteration, one does
C not need the grid based interpolation, since
C the myopic loss can be evaluation directly
C
       IF(INTER.EQ.1) THEN
        IF(ITVAL.EQ.1) THEN
C
C         IF(IADJUST.EQ.1) THEN
C          VBUP = ADJUST+(SIGMA + (BUP3-TARGET)**2 
C     X        * BUP2 /(BUP2+BUP1**2))/(1-DELTA)
C         ELSE
           VBUP = (SIGMA + (BUP3-TARGET)**2 
     X        * BUP2 /(BUP2+BUP1**2))
C         ENDIF
        GOTO 125
        ENDIF
       ENDIF
C do interpolation starting with the second
C value function iteration
C
C           WRITE(*,*) 'INTERPOLATE'
C 
C evaluate V at the corners of the gridsquare
C
           V111 = V(IP11,IP21,IP31,1)
           V211 = V(IP12,IP21,IP31,1)
           V221 = V(IP12,IP22,IP31,1)
           V121 = V(IP11,IP22,IP31,1)
           V112 = V(IP11,IP21,IP32,1)
           V212 = V(IP12,IP21,IP32,1)
           V222 = V(IP12,IP22,IP32,1)
           V122 = V(IP11,IP22,IP32,1)

C            WRITE(*,*)
C
C Interpolating the value of the value function at the updated
C beliefs (state variables). The interpolation is based on 
C squares: piecewise bilinear approximation
C
       VBUP= V111*(1-W1)*(1-W2)*(1-W3)+V211*W1*(1-W2)*(1-W3)
     X       +V221*W1*W2*(1-W3)+V121*(1-W1)*W2*(1-W3)
     X       +V112*(1-W1)*(1-W2)*W3+V212*W1*(1-W2)*W3
     X       +V222*W1*W2*W3+V122*(1-W1)*W2*W3
C
 125         CONTINUE
C
C calculate the integral using VBUP and summing over weights
C and nodes according to 10point Gaussian quadrature
C
         EV = EV + (1.0d0/SQRT(3.141592654d0))
     X              *VBUP*WEI(K)    
C
C
C end of loop for integration
C
        IF (LOOK.EQ.3) THEN
         WRITE(*,*)
         WRITE(*,*) 'K=',K,IP1,IP2,IP3
         WRITE(*,*)
         WRITE(*,*) 'BUP1,IP11,IP12,PG1(IP11),PG1(IP12)'
         WRITE(*,*) BUP1,IP11,IP12,PG1(IP11),PG1(IP12)
         WRITE(*,*)
         WRITE(*,*) 'BUP2,IP21,IP22,PG2(IP21),PG2(IP22)'
         WRITE(*,*) BUP2,IP21,IP22,PG2(IP21),PG2(IP22)
         WRITE(*,*)
         WRITE(*,*) 'BUP3,IP31,IP32,PG3(IP31),PG3(IP32)'
         WRITE(*,*) BUP3,IP31,IP32,PG3(IP31),PG3(IP32)
         WRITE(*,*)
         WRITE(*,*) 'W1,W2,W3 ',W1,W2,W3
         WRITE(*,*)
         WRITE(*,*) 'V AT CORNERS: V111,V211,V221,V121'
         WRITE(*,*) V111,V211,V221,V121
         WRITE(*,*) 'V AT CORNERS: V112,V212,V222,V122'
         WRITE(*,*) V112,V212,V222,V122
         WRITE(*,*) 'VBUP=', VBUP
         WRITE(*,*) 'EV=',EV
         WRITE(*,*)
        ENDIF
C
 100      CONTINUE
C
C         
C evaluate the right-hand side of the functional equation FEVAL
C
        FEVAL = SIGMA + (RHO*PG3(IP3)-TARGET)**2 
     X    + 2.0d0 * (RHO*PG3(IP3)-TARGET) * PG1(IP1) * X +
     X      (PG2(IP2)+SIGMAETA
     x     + PG1(IP1)**2) * X**2 + DELTA * EV
C
        IF(LOOK.GE.3) THEN
         WRITE(*,*) 'I1,IP2,IP3,X,FEVAL,EV'
          WRITE(*,*) IP1,IP2,IP3,X,FEVAL,EV
          WRITE(*,*)
          WRITE(*,*) '****************************************'
        ENDIF
C
        RETURN
C
        END   
C
C
C***********************************************************************
C
        SUBROUTINE LOCATE(PP,IPG,BUP,LOC)
C
C This subroutine implements a search on a grid for the two gridpoints
C closest to a given value BUP. It finds the closest lower neighbor
C LL=LOC and the closest upper neighbor LU=LOC+1. The search is
C conducted by means of the bisection method. The routine also 
C recognizes if the given value BUP is outside the defined grid.
C If BUP is smaller than the lowest gridpoint LL=0. If the BUP is 
C greater than the largest gridpoint, then LL=IPG
C
C
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        DOUBLE PRECISION BUP,PP(IPG)
        INTEGER IPG,LOC,LL,LM,LU
C
        LL=0
        LU=IPG+1
 10     IF(LU-LL.GT.1) THEN
               LM=(LU+LL)/2
               IF((PP(IPG).GT.PP(1)).EQV.(BUP.GT.PP(LM))) THEN
                 LL=LM
               ELSE
                 LU=LM
               ENDIF
C               write(*,*) 'll,lu,lm',ll,lu,lm
        GOTO 10
        ENDIF
C
        LOC=LL
C
        RETURN
        END
C 
C
C***********************************************************************
C
        DOUBLE PRECISION FUNCTION GOLDEN(AX,BX,CX,F,TOL,XMIN)
C
C routine for golden section search for minimum, copied from Numerical
C Recipes in Fortran
C Given a function f, and given a bracketing triplet of absicissas ax,
C bx,cx,(such that bx is between ax and cx and f(bx) is less than both
C f(ax) and f(cx)), this routine performs a golden section search for the
C minimum , isolating it to a fractional precision of about tol. The
C abscissa of the minimum is returned as xmin, and the minimum function
C value is returned as golden, the returned function value
C Parameters: the golden ratios.
C
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        PARAMETER (R=.61803399,C=1.-R)
C        PARAMETER (R=.5,C=1.-R)
        DOUBLE PRECISION AX,BX,CX,TOL,XMIN,F
        DOUBLE PRECISION F1,F2,X0,X1,X2,X3
        INTEGER LOOK
        EXTERNAL F
C
        COMMON /LLOOK/ LOOK
C
C at any given time we will keep track of four points, x0,x1,x2,x3
C
        X0=AX
        X3=CX
C
C make x0 to x1 the smaller segment and fill in the new point to
C be tried.
C
        CH1=ABS(CX-BX)
        CH2=ABS(BX-AX)
C
        IF(CH1.GT.CH2) THEN
              X1=BX
              X2=BX+C*(CX-BX)
        ELSE
              X2=BX
              X1=BX-C*(BX-AX)
        ENDIF
C
C the initial function evaluations. Note that we never need to evaluate
C the function at the original endpoints
C
C        X=X1
        F1=F(X1)
C        X=X2
        F2=F(X2)
C
C
C Do-while loop: we keep returning here, one possible outcome is 
C housekeeping, and a new function evaluation, the other outcome,
C ..
C
C 
 10     IF(ABS(X3-X0).GT.TOL*(ABS(X1)+ABS(X2))) THEN
           IF(F2.LT.F1) THEN
                X0=X1
                X1=X2
                X2=R*X1+C*X3
                F1=F2
C                X=X2
                if(look.eq.4) then
                  write(*,*) 'x0-3',x0,x1,x2,X3
                endif
                F2=F(X2)
           ELSE
                X3=X2
                X2=X1
                X1=R*X2+C*X0
                F2=F1
C                X=X1
                F1=F(X1)
                if(look.eq.4) then
                write(*,*) 'x0-3',x0,x1,x2,X3
                endif
           ENDIF
           GOTO 10
        ENDIF
C
        IF(F1.LT.F2) THEN
            GOLDEN=F1
            XMIN=X1
                if(look.eq.2) then
                  look=3
                  write(*,*) 'x0-3',x0,x1,x2,X3
                  write(*,*) ' in golden subroutine'
                test=F(x1)
                endif
        ELSE
            GOLDEN=F2
            XMIN=X2
                if(look.eq.2) then
                  look=3
                  write(*,*) 'x0-3',x0,x1,x2,X3
                  write(*,*) ' in golden subroutine'
                test=F(x2)
                endif
        ENDIF 
C     
             if(look.eq.4) then
               write(*,*) 'xming',xmin
             endif
C
        RETURN
        END














